#include <ctype.h>
#include <iostream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
using namespace std;

#include "calendar.h"
#include "time.h"



/*
 * コンストラクタ
 */
Calendar::Calendar()
{

    // （角度の）度からラジアンに変換する係数の定義
    dK = PI / 180.0;

    // メッセージ定義
    defMsg();

    // 曜日データ定義
    defWeek();

    // 六曜データ定義
    defRokuyo();

    // 干支データ定義
    defKanshi();

    // 二十四節気データ定義
    defSekki24();

    // 節句データ定義
    defSekku();

    // 雑節データ定義
    defZassetsu();

    // 休日データ定義
    defHoliday();
}

/*
 * 引数チェック
 */
bool Calendar::checkArg(int argc, char *argv[])
{
    double dJd;                            // ユリウス通日
    time_t now;                            // 現在日時
    struct tm* pnow;                       // 日時構造体

    try {
        // オプション初期値
        pcOption = "abcdefghijkl";

        // 現在年月日取得
        now = time(NULL);
        pnow = localtime(&now);
        iYear  = pnow->tm_year + 1900;
        iMonth = pnow->tm_mon + 1;
        iDay   = pnow->tm_mday;

        if (argc == 1) {
            // 引数なし
        } else if (argc == 2) {
            // 引数１個
            if (strlen(argv[1]) == 8) {
                // 全部半角数字なら年・月・日再設定
                if (isNumeric(argv[1])) {
                    char cYear[5], cMonth[5], cDay[5];
                    cYear[0]  = '\0';
                    cMonth[0] = '\0';
                    cDay[0]   = '\0';
                    for (int i = 0; i < 4; i++)
                        strncat(cYear,  &argv[1][i], 1);
                    for (int i = 4; i < 6; i++)
                        strncat(cMonth, &argv[1][i], 1);
                    for (int i = 6; i < 8; i++)
                        strncat(cDay,   &argv[1][i], 1);
                    iYear  = atoi(cYear);
                    iMonth = atoi(cMonth);
                    iDay   = atoi(cDay);
                } else {
                    pcErrMsg = pacMsg[0];
                    return false;
                }
            } else {
                // a - l の半角英字ならオプション再設定
                if (isChar(argv[1])) {
                    pcOption = argv[1];
                    // cout << pcOption << endl;
                } else {
                    pcErrMsg = pacMsg[0];
                    return false;
                }
            }
        } else if (argc >= 3) {
            // 引数２個
            // 第１引数 : a - l の半角英字ならオプション再設定
            if (isChar(argv[1])) {
                pcOption = argv[1];
            } else {
                pcErrMsg = pacMsg[0];
                return false;
            }

            // 第２引数 : 全部半角数字なら年・月・日再設定
            if (strlen(argv[2]) == 8) {
                if (isNumeric(argv[2])) {
                    char cYear[5], cMonth[5], cDay[5];
                    cYear[0]  = '\0';
                    cMonth[0] = '\0';
                    cDay[0]   = '\0';
                    for (int i = 0; i < 4; i++)
                        strncat(cYear,  &argv[2][i], 1);
                    for (int i = 4; i < 6; i++)
                        strncat(cMonth, &argv[2][i], 1);
                    for (int i = 6; i < 8; i++)
                        strncat(cDay,   &argv[2][i], 1);
                    iYear  = atoi(cYear);
                    iMonth = atoi(cMonth);
                    iDay   = atoi(cDay);
                } else {
                    pcErrMsg = pacMsg[0];
                    return false;
                }
            } else {
                pcErrMsg = pacMsg[0];
                return false;
            }
        }

        // 日付妥当性チェック
        if (!checkDate(iYear, iMonth, iDay)) {
            pcErrMsg = pacMsg[1];
            return false;
        }

        // グレゴリオ暦からユリウス通日を計算
        dJd = dblGc2Jd(iYear, iMonth, iDay, 0, 0, 0);

        // ユリウス通日から曜日を計算
        iWeekDay = intCalcDayOfWeek(dJd);

        // 結果格納構造体へ格納
        stRes.iYear    = iYear;
        stRes.iMonth   = iMonth;
        stRes.iDay     = iDay;
        stRes.iWeekDay = iWeekDay;
        stRes.dJd      = dJd;

        return true;
    } catch (...) {
        pcErrMsg = pacMsg[2];
        return false;
    }
}

/*
 * エラーメッセージ取得
 */
const char* Calendar::getErrMsg()
{
    return pcErrMsg;
}

/*
 * 数値チェック
 */
bool Calendar::isNumeric(char *num)
{
    for (int i = 0; i < strlen(num); i++) {
        if (isdigit(num[i]) == 0)
            return false;
    }

    return true;
}

/*
 * 半角英字チェック
 * ( a - l 以外は false )
 */
bool Calendar::isChar(char *opt)
{
    for (int i = 0; i < strlen(opt); i++) {
        if (opt[i] < 97 || 108 < opt[i])
            return false;
    }

    return true;
}

/*
 * 日付妥当性チェック
 */
bool Calendar::checkDate(int y, int m, int d)
{
    return (m >= 1 && m <= 12) && ( d >= 1 && d <= daysInMonth(y, m));
}

/*
 * 閏年(leap year)チェック
 */
bool Calendar::isLeapYear(int y)
{
    return (y % 400 == 0 || y % 100 != 0 && y % 4 == 0);
}

/*
 * 指定された月の日数を返す
 * 入力された月が不正（１月～12月の範囲外）であれば０を返す
 */
int Calendar::daysInMonth(int y, int m)
{
    const int nDaysInMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

    if (m < 1 || 12 < m)
        return 0;
    if (m == 2 && isLeapYear(y))
        return 29;

    return nDaysInMonth[m - 1];
}

/*
 * メッセージ定義
 */
void Calendar::defMsg()
{
    // 文字列定数定義
    pacMsg[ 0] = "引数指定 : [ 半角英字 ( abcdefghijkl ) ] [ 半角数字(８桁) ] ";
    pacMsg[ 1] = "引数指定 : 妥当な日付ではありません。";
    pacMsg[ 2] = "引数チェック失敗！";
    pacMsg[ 3] = "二十四節気計算失敗！";
    pacMsg[ 4] = "雑節計算失敗！";
    pacMsg[ 5] = "休日計算失敗！";
    pacMsg[ 6] = "旧暦計算失敗！";
    pacMsg[ 7] = "世界時計算失敗！";
    pacMsg[ 8] = "二分二至計算失敗！";
    pacMsg[ 9] = "旧暦計算(コア部分)失敗！";
    pacMsg[10] = "六曜計算失敗！";
    pacMsg[11] = "干支計算失敗！";
    pacMsg[12] = "節句計算失敗！";
    pacMsg[13] = "ユリウス日計算失敗！";
    pacMsg[14] = "黄経(太陽)計算失敗！";
    pacMsg[15] = "黄経(月)計算失敗！";
    pacMsg[16] = "月齢(正午)計算失敗！";
    pacMsg[17] = "角度正規化失敗！";
    pacMsg[18] = "曜日計算失敗！";
    pacMsg[19] = "朔計算失敗！";
    pacMsg[20] = "二十四節気取得処理失敗！";
    pacMsg[21] = "雑節取得処理失敗！";
    pacMsg[22] = "休日取得処理失敗！";
    pacMsg[23] = "旧暦取得処理失敗！";
    pacMsg[24] = "結果出力処理失敗！";
    pacMsg[25] = "ユリウス通日取得処理失敗！";
}

/*
 * 曜日データ定義
 */
void Calendar::defWeek()
{
    pacWeek[0] = "日";
    pacWeek[1] = "月";
    pacWeek[2] = "火";
    pacWeek[3] = "水";
    pacWeek[4] = "木";
    pacWeek[5] = "金";
    pacWeek[6] = "土";
}

/*
 * 六曜データ定義
 */
void Calendar::defRokuyo()
{
    pacRokuyo[0] = "大安";
    pacRokuyo[1] = "赤口";
    pacRokuyo[2] = "先勝";
    pacRokuyo[3] = "友引";
    pacRokuyo[4] = "先負";
    pacRokuyo[5] = "仏滅";
}

/*
 * 干支データ定義
 */
void Calendar::defKanshi()
{
    // 干データ
    pacKan[ 0] = "甲";
    pacKan[ 1] = "乙";
    pacKan[ 2] = "丙";
    pacKan[ 3] = "丁";
    pacKan[ 4] = "戊";
    pacKan[ 5] = "己";
    pacKan[ 6] = "庚";
    pacKan[ 7] = "辛";
    pacKan[ 8] = "壬";
    pacKan[ 9] = "癸";
    // 支データ
    pacShi[ 0] = "子";
    pacShi[ 1] = "丑";
    pacShi[ 2] = "寅";
    pacShi[ 3] = "卯";
    pacShi[ 4] = "辰";
    pacShi[ 5] = "巳";
    pacShi[ 6] = "午";
    pacShi[ 7] = "未";
    pacShi[ 8] = "申";
    pacShi[ 9] = "酉";
    pacShi[10] = "戌";
    pacShi[11] = "亥";
}

/*
 * 二十四節気データ定義
 */
void Calendar::defSekki24()
{
    pacSekki24[ 0] = "春分";
    pacSekki24[ 1] = "清明";
    pacSekki24[ 2] = "穀雨";
    pacSekki24[ 3] = "立夏";
    pacSekki24[ 4] = "小満";
    pacSekki24[ 5] = "芒種";
    pacSekki24[ 6] = "夏至";
    pacSekki24[ 7] = "小暑";
    pacSekki24[ 8] = "大暑";
    pacSekki24[ 9] = "立秋";
    pacSekki24[10] = "処暑";
    pacSekki24[11] = "白露";
    pacSekki24[12] = "秋分";
    pacSekki24[13] = "寒露";
    pacSekki24[14] = "霜降";
    pacSekki24[15] = "立冬";
    pacSekki24[16] = "小雪";
    pacSekki24[17] = "大雪";
    pacSekki24[18] = "冬至";
    pacSekki24[19] = "小寒";
    pacSekki24[20] = "大寒";
    pacSekki24[21] = "立春";
    pacSekki24[22] = "雨水";
    pacSekki24[23] = "啓蟄";
}

/*
 * 節句データ定義
 */
void Calendar::defSekku()
{
    astSekku[0].iMonth = 1;
    astSekku[0].iDay   = 7;
    astSekku[0].cName  = "人日";
    astSekku[1].iMonth = 3;
    astSekku[1].iDay   = 3;
    astSekku[1].cName  = "上巳";
    astSekku[2].iMonth = 5;
    astSekku[2].iDay   = 5;
    astSekku[2].cName  = "端午";
    astSekku[3].iMonth = 7;
    astSekku[3].iDay   = 7;
    astSekku[3].cName  = "七夕";
    astSekku[4].iMonth = 9;
    astSekku[4].iDay   = 9;
    astSekku[4].cName  = "重陽";
}

/*
 * 雑節データ定義
 */
void Calendar::defZassetsu()
{
    pacZassetsu[ 0] = "節分";
    pacZassetsu[ 1] = "彼岸入(春)";
    pacZassetsu[ 2] = "彼岸(春)";
    pacZassetsu[ 3] = "彼岸明(春)";
    pacZassetsu[ 4] = "社日(春)";
    pacZassetsu[ 5] = "土用入(春)";
    pacZassetsu[ 6] = "八十八夜";
    pacZassetsu[ 7] = "入梅";
    pacZassetsu[ 8] = "半夏生";
    pacZassetsu[ 9] = "土用入(夏)";
    pacZassetsu[10] = "二百十日";
    pacZassetsu[11] = "二百二十日";
    pacZassetsu[12] = "彼岸入(秋)";
    pacZassetsu[13] = "彼岸(秋)";
    pacZassetsu[14] = "彼岸明(秋)";
    pacZassetsu[15] = "社日(秋)";
    pacZassetsu[16] = "土用入(秋)";
    pacZassetsu[17] = "土用入(冬)";
}

/*
 * 休日データ定義
 */
void Calendar::defHoliday()
{
    astHoliday[ 0].iKey   = 0;
    astHoliday[ 0].iMonth = 1;
    astHoliday[ 0].iDay   = 1;
    astHoliday[ 0].iKbn   = 99;
    astHoliday[ 0].cName  = "元日";
    astHoliday[ 1].iKey   = 1;
    astHoliday[ 1].iMonth = 1;
    astHoliday[ 1].iDay   = 99;
    astHoliday[ 1].iKbn   = 21;
    astHoliday[ 1].cName  = "成人の日";
    astHoliday[ 2].iKey   = 2;
    astHoliday[ 2].iMonth = 2;
    astHoliday[ 2].iDay   = 11;
    astHoliday[ 2].iKbn   = 99;
    astHoliday[ 2].cName  = "建国記念の日";
    astHoliday[ 3].iKey   = 3;
    astHoliday[ 3].iMonth = 3;
    astHoliday[ 3].iDay   = 99;
    astHoliday[ 3].iKbn   = 80;
    astHoliday[ 3].cName  = "春分の日";
    astHoliday[ 4].iKey   = 4;
    astHoliday[ 4].iMonth = 4;
    astHoliday[ 4].iDay   = 29;
    astHoliday[ 4].iKbn   = 99;
    astHoliday[ 4].cName  = "昭和の日";
    astHoliday[ 5].iKey   = 5;
    astHoliday[ 5].iMonth = 5;
    astHoliday[ 5].iDay   = 3;
    astHoliday[ 5].iKbn   = 99;
    astHoliday[ 5].cName  = "憲法記念日";
    astHoliday[ 6].iKey   = 6;
    astHoliday[ 6].iMonth = 5;
    astHoliday[ 6].iDay   = 4;
    astHoliday[ 6].iKbn   = 99;
    astHoliday[ 6].cName  = "みどりの日";
    astHoliday[ 7].iKey   = 7;
    astHoliday[ 7].iMonth = 5;
    astHoliday[ 7].iDay   = 5;
    astHoliday[ 7].iKbn   = 99;
    astHoliday[ 7].cName  = "こどもの日";
    astHoliday[ 8].iKey   = 8;
    astHoliday[ 8].iMonth = 7;
    astHoliday[ 8].iDay   = 99;
    astHoliday[ 8].iKbn   = 31;
    astHoliday[ 8].cName  = "海の日";
    astHoliday[ 9].iKey   = 9;
    astHoliday[ 9].iMonth = 9;
    astHoliday[ 9].iDay   = 99;
    astHoliday[ 9].iKbn   = 31;
    astHoliday[ 9].cName  = "敬老の日";
    astHoliday[10].iKey   = 10;
    astHoliday[10].iMonth = 9;
    astHoliday[10].iDay   = 99;
    astHoliday[10].iKbn   = 81;
    astHoliday[10].cName  = "秋分の日";
    astHoliday[11].iKey   = 11;
    astHoliday[11].iMonth = 10;
    astHoliday[11].iDay   = 99;
    astHoliday[11].iKbn   = 21;
    astHoliday[11].cName  = "体育の日";
    astHoliday[12].iKey   = 12;
    astHoliday[12].iMonth = 11;
    astHoliday[12].iDay   = 3;
    astHoliday[12].iKbn   = 99;
    astHoliday[12].cName  = "文化の日";
    astHoliday[13].iKey   = 13;
    astHoliday[13].iMonth = 11;
    astHoliday[13].iDay   = 23;
    astHoliday[13].iKbn   = 99;
    astHoliday[13].cName  = "勤労感謝の日";
    astHoliday[14].iKey   = 14;
    astHoliday[14].iMonth = 12;
    astHoliday[14].iDay   = 23;
    astHoliday[14].iKbn   = 99;
    astHoliday[14].cName  = "天皇誕生日";
    astHoliday[15].iKey   = 90;
    astHoliday[15].iMonth = 99;
    astHoliday[15].iDay   = 99;
    astHoliday[15].iKbn   = 99;
    astHoliday[15].cName  = "国民の休日";
    astHoliday[16].iKey   = 91;
    astHoliday[16].iMonth = 99;
    astHoliday[16].iDay   = 99;
    astHoliday[16].iKbn   = 99;
    astHoliday[16].cName  = "振替休日";
}

/*
 * ユリウス通日取得処理
 */
bool Calendar::blnGetJd()
{
    try {
        // グレゴリオ暦からユリウス通日を計算
        dJd = dblGc2Jd(iYear, iMonth, iDay, 0, 0, 0);

        // 値セット
        stRes.dJd = dJd;

        return true;
    } catch (...) {
        return false;
    }
}
/*
 * 二十四節気取得処理
 */
bool Calendar::blnGetSekki24()
{
    try {
        // 二十四節気計算
        if (!blnCalcSekki24(dJd)) {
            pcErrMsg = pacMsg[3];
            return false;
        }

        // 値セット
        stRes.iSekki24 = iSekki24;

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  引数で与えられたユリウス通日(JD)の二十四節気を計算する
 *    [ 引数 ]
 *      jd : ユリウス通日
 *    [ 戻り値 ]
 *      真偽値
 *
 *      iSekki24 : 二十四節気の黄経 ( 太陽 )
 *                 ( 二十四節気でなければ、999 )
 *    ※基点ユリウス日(2451545)：2000/1/2/0/0/0(年/月/日/時/分/秒…世界時)
 ******************************************************************************/
bool Calendar::blnCalcSekki24(double dJd)
{
    double tm1, tm2, t;                  // 時刻
    int    lsun_today, lsun_tomorrow;    // 黄経(太陽)
    int    lsun_today0, lsun_tomorrow0;  // 黄経(太陽)丸め値

    try {
        // 時刻引数を分解
        tm1  = floor(dJd);  // 整数部分
        tm2  = dJd - tm1;   // 小数部分
        tm2 -= 9 / 24.0;    // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )

        // 計算対象日の太陽の黄経
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_today = dblCalcLongitudeSun(t);

        // 計算対象日の翌日のユリウス日
        dJd += 1;           // 1日プラス
        tm1  = floor(dJd);  // 整数部分
        tm2  = dJd - tm1;   // 小数部分
        tm2 -= 9 / 24.0;    // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )

        // 計算対象日の翌日のの太陽の黄経
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_tomorrow = dblCalcLongitudeSun(t);

        lsun_today0    = 15 * (double)(int)(lsun_today / 15.0);
        lsun_tomorrow0 = 15 * (double)(int)(lsun_tomorrow / 15.0);
        if (lsun_today0 != lsun_tomorrow0) {
            iSekki24 = lsun_tomorrow0;
        } else {
            iSekki24 = 999;
        }

        return true;
    } catch (...) {
        return false;
    }
}

/*
 * 雑節取得処理
 */
bool Calendar::blnGetZassetsu()
{
    try {
        // 雑節計算
        if (!blnCalcZassetsu(dJd)) {
            pcErrMsg = pacMsg[4];
            return false;
        }

        // 値セット
        stRes.iZassetsu1 = iZassetsu1;
        stRes.iZassetsu2 = iZassetsu2;

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  引数からの雑節を求める
 *    [ 引数 ]
 *      jd        : ユリウス通日
 *    [ 戻り値 ]
 *      真偽値
 *
 *      iZassetsu1 : 雑節のｺｰﾄﾞ
 *      iZassetsu2 : 雑節のｺｰﾄﾞ(同日に複数の雑節がある場合)
 *    ※基点ユリウス日(2451545)：2000/1/2/0/0/0(年/月/日/時/分/秒…世界時)
 ******************************************************************************/
bool Calendar::blnCalcZassetsu(double dJd)
{
    double tm1, tm2, t, dJd2;
    double lsun_today, lsun_tomorrow;
    double lsun_today0, lsun_tomorrow0;
    double lsun_before_5, lsun_before_4;
    double lsun_after_5, lsun_after_6;

    try {
        // 社日は他の雑節と重なる可能性があるので、
        // 重なる場合はzassetu_2を使用する。
        iZassetsu1 = 99;
        iZassetsu2 = 99;

        // 計算対象日の太陽の黄経
        tm1  = floor(dJd);               // 整数部分
        tm2  = dJd - tm1;                // 小数部分
        tm2 -= 9 / 24.0;                 // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_today = dblCalcLongitudeSun(t);

        // 計算対象日の翌日の太陽の黄経
        dJd2 = dJd + 1;                  // 1日プラス
        tm1  = floor(dJd2);              // 整数部分
        tm2  = dJd2 - tm1;               // 小数部分
        tm2 -= 9 / 24.0;                 // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_tomorrow = dblCalcLongitudeSun(t);

        // 計算対象日の5日前の太陽の黄経(社日計算用)
        dJd2 = dJd - 5;                  // 5日マイナス
        tm1  = floor(dJd2);              // 整数部分
        tm2  = dJd2 - tm1;               // 小数部分
        tm2 -= 9 / 24.0;                 // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_before_5 = dblCalcLongitudeSun(t);

        // 計算対象日の4日前の太陽の黄経(社日計算用)
        dJd2 = dJd - 4;                  // 4日マイナス
        tm1  = floor(dJd2);              // 整数部分
        tm2  = dJd2 - tm1;               // 小数部分
        tm2 -= 9 / 24.0;                 // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_before_4 = dblCalcLongitudeSun(t);

        // 計算対象日の5日後の太陽の黄経(社日計算用)
        dJd2 = dJd + 5;                  // 5日プラス
        tm1  = floor(dJd2);              // 整数部分
        tm2  = dJd2 - tm1;               // 小数部分
        tm2 -= 9 / 24.0;                 // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_after_5 = dblCalcLongitudeSun(t);

        // 計算対象日の6日後の太陽の黄経(社日計算用)
        dJd2  = dJd + 6;                 // 6日プラス
        tm1  = floor(dJd2);              // 整数部分
        tm2  = dJd2 - tm1;               // 小数部分
        tm2 -= 9 / 24.0;                 // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )
        t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
        lsun_after_6 = dblCalcLongitudeSun(t);

        // 太陽の黄経の整数部分( 土用, 入梅, 半夏生 計算用 )
        lsun_today0   = (double)(int)(lsun_today);
        lsun_tomorrow0 = (double)(int)(lsun_tomorrow);

        // ====< ここから各種雑節計算 >====

        // 0:節分 ( 立春の前日 )
        if (!blnCalcSekki24(dJd + 1)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 315) {  // 立春の黄経(太陽)
            iZassetsu1 = 0;
        }

        // 1:彼岸入（春） ( 春分の日の3日前 )
        if (!blnCalcSekki24(dJd + 3)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
            iZassetsu1 = 1;
        }

        // 2:彼岸（春） ( 春分の日 )
        if (!blnCalcSekki24(dJd)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
            iZassetsu1 = 2;
        }

        // 3:彼岸明（春） ( 春分の日の3日後 )
        if (!blnCalcSekki24(dJd - 3)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
            iZassetsu1 = 3;
        }

        // 4:社日（春） ( 春分の日に最も近い戊(つちのえ)の日 )
        // * 計算対象日が戊の日の時、
        //   * 4日後までもしくは4日前までに春分の日がある時、
        //       この日が社日
        //   * 5日後が春分の日の時、
        //       * 春分点(黄経0度)が午前なら
        //           この日が社日
        //       * 春分点(黄経0度)が午後なら
        //           この日の10日後が社日
        if (((int)dJd % 10) == 4) {  // 戊の日
            // [ 当日から4日後 ]
            for (int i = 0; i <= 4; i++ ) {
                if (!blnCalcSekki24(dJd + i)) {
                    pcErrMsg = pacMsg[3];
                    return false;
                }
                if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 4;
                    } else {
                        iZassetsu2 = 4;
                    }
                    break;
                }
            }
            // [ 1日前から4日前 ]
            for (int i = 1; i <= 4; i++) {
                if (!blnCalcSekki24(dJd - i)) {
                    pcErrMsg = pacMsg[3];
                    return false;
                }
                if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 4;
                    } else {
                        iZassetsu2 = 4;
                    }
                    break;
                }
            }
            // [ 5日後 ]
            if (!blnCalcSekki24(dJd + 5)) {
                pcErrMsg = pacMsg[3];
                return false;
            }
            if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
                // 春分の日の黄経(太陽)と翌日の黄経(太陽)の中間点が
                // 0度(360度)以上なら、春分点が午前と判断
                if ((lsun_after_5 + lsun_after_6 + 360) / 2.0 >= 360) {
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 4;
                    } else {
                        iZassetsu2 = 4;
                    }
                }
            }
            // [ 5日前 ]
            if (!blnCalcSekki24(dJd - 5)) {
                pcErrMsg = pacMsg[3];
                return false;
            }
            if (iSekki24 == 0) {  // 春分の日の黄経(太陽)
                // 春分の日の黄経(太陽)と翌日の黄経(太陽)の中間点が
                // 0度(360度)未満なら、春分点が午後と判断
                if ((lsun_before_4 + lsun_before_5 + 360) / 2.0 < 360) {
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 4;
                    } else {
                        iZassetsu2 = 4;
                    }
                }
            }
        }

        // 5:土用入（春） ( 黄経(太陽) = 27度 )
        if (lsun_today0 != lsun_tomorrow0) {
            if (lsun_tomorrow0 == 27) {
                iZassetsu1 = 5;
            }
        }

        // 6:八十八夜 ( 立春から88日目(87日後) )
        if (!blnCalcSekki24(dJd - 87)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 315) {  // 立春の黄経(太陽)
            iZassetsu1 = 6;
        }

        // 7:入梅 ( 黄経(太陽) = 80度 )
        if (lsun_today0 != lsun_tomorrow0) {
            if (lsun_tomorrow0 == 80) {
                iZassetsu1 = 7;
            }
        }

        // 8:半夏生  ( 黄経(太陽) = 100度 )
        if (lsun_today0 != lsun_tomorrow0) {
            if (lsun_tomorrow0 == 100) {
                iZassetsu1 = 8;
            }
        }

        // 9:土用入（夏） ( 黄経(太陽) = 117度 )
        if (lsun_today0 != lsun_tomorrow0) {
            if (lsun_tomorrow0 == 117) {
                iZassetsu1 = 9;
            }
        }

        // 10:二百十日 ( 立春から210日目(209日後) )
        if (!blnCalcSekki24(dJd - 209)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 315) {   // 立春の黄経(太陽)
            iZassetsu1 = 10;
        }

        // 11:二百二十日 ( 立春から220日目(219日後) )
        if (!blnCalcSekki24(dJd - 219)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 315) {  // 立春の黄経(太陽)
            iZassetsu1 = 11;
        }

        // 12:彼岸入（秋） ( 秋分の日の3日前 )
        if (!blnCalcSekki24(dJd + 3)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 180) {  // 秋分の日の黄経(太陽)
            iZassetsu1 = 12;
        }

        // 13:彼岸（秋）   ( 秋分の日 )
        if (!blnCalcSekki24(dJd)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 180) {  // 秋分の日の黄経(太陽)
            iZassetsu1 = 13;
        }

        // 14:彼岸明（秋） ( 秋分の日の3日後 )
        if (!blnCalcSekki24(dJd - 3)) {
            pcErrMsg = pacMsg[3];
            return false;
        }
        if (iSekki24 == 180) {  // 春分の日の黄経(太陽)
            iZassetsu1 = 14;
        }

        // 15:社日（秋） ( 秋分の日に最も近い戊(つちのえ)の日 )
        // * 計算対象日が戊の日の時、
        //   * 4日後までもしくは4日前までに秋分の日がある時、
        //       この日が社日
        //   * 5日後が秋分の日の時、
        //       * 秋分点(黄経180度)が午前なら
        //           この日が社日
        //       * 秋分点(黄経180度)が午後なら
        //           この日の10日後が社日
        if (((int)dJd % 10) == 4) {  // 戊の日
            // [ 当日から4日後 ]
            for (int i = 0; i <= 4; i++) {
                if (!blnCalcSekki24(dJd + i)) {
                    pcErrMsg = pacMsg[3];
                    return false;
                }
                if (iSekki24 == 180) {  // 秋分の日の黄経(太陽)
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 15;
                    } else {
                        iZassetsu2 = 15;
                    }
                    break;
                }
            }
            // [ 1日前から4日前 ]
            for (int i = 1; i <= 4; i++) {
                if (!blnCalcSekki24(dJd - i)) {
                    pcErrMsg = pacMsg[3];
                    return false;
                }
                if (iSekki24 == 180) {  // 秋分の日の黄経(太陽)
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 15;
                    } else {
                        iZassetsu2 = 15;
                    }
                    break;
                }
            }
            // [ 5日後 ]
            if (!blnCalcSekki24(dJd + 5)) {
                pcErrMsg = pacMsg[3];
                return false;
            }
            if (iSekki24 == 180) {  // 秋分の日の黄経(太陽)
                // 秋分の日の黄経(太陽)と翌日の黄経(太陽)の中間点が
                // 180度以上なら、秋分点が午前と判断
                if ((lsun_after_5 + lsun_after_6) / 2.0 >= 180) {
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 15;
                    } else {
                        iZassetsu2 = 15;
                    }
                }
            }
            // [ 5日前 ]
            if (!blnCalcSekki24(dJd - 5)) {
                pcErrMsg = pacMsg[3];
                return false;
            }
            if (iSekki24 == 180) {  // 秋分の日の黄経(太陽)
                // 秋分の日の黄経(太陽)と翌日の黄経(太陽)の中間点が
                // 180度未満なら、秋分点が午後と判断
                if ((lsun_before_4 + lsun_before_5) / 2.0 < 180) {
                    if (iZassetsu1 == 99) {
                        iZassetsu1 = 15;
                    } else {
                        iZassetsu2 = 15;
                    }
                }
            }
        }

        // 16:土用入（秋） ( 黄経(太陽) = 207度 )
        if (lsun_today0 != lsun_tomorrow0) {
            if (lsun_tomorrow0 == 207) {
                iZassetsu1 = 16;
            }
        }

        // 17:土用入（冬） ( 黄経(太陽) = 297度 )
        if (lsun_today0 != lsun_tomorrow0) {
            if (lsun_tomorrow0 == 297) {
                iZassetsu1 = 17;
            }
        }

        return true;
    } catch (...) {
        return false;
    }
}

/*
 * 休日取得処理
 */
bool Calendar::blnGetHoliday()
{
    try {
        // 休日計算
        if (!blnCalcHoliday(dJd)) {
            pcErrMsg = pacMsg[5];
            return false;
        }

        // 値セット
        stRes.iHoliday = iHoliday;

        return true;
    } catch (...) {
        return false;
    }
}

/*
 * 休日計算
 */
bool Calendar::blnCalcHoliday(double dJd)
{
    double dWkJd, dJdNext;
    int    iYobi, iYobiNext;
    double dNibunJd;
    int    iSize0 = 0;
    int    iSize1 = 0;
    int    iSize2 = 0;
    int    iSize  = 0;
    struct stHOLIDAY0 {
        int    iMonth;
        int    iDay;
        int    iKey;
        double dJd;
        int    iYobi;
    };                             // 休日データ構造体
    stHOLIDAY0 astHoliday0[20];    // 休日データ(変動祝日)配列
    stHOLIDAY0 astHoliday1[20];    // 休日データ(国民の休日)配列
    stHOLIDAY0 astHoliday2[20];    // 休日データ(振替休日)配列

    try {
        // 変動の祝日の日付･曜日を計算 ( 振替休日,国民の休日を除く )
        for (int i = 0; i < sizeof(astHoliday) / sizeof(astHoliday[0]); i++) {
            if (astHoliday[i].iMonth != 99) {
                if (astHoliday[i].iDay != 99) {  // 月日が既定のもの
                    // グレゴリオ暦からユリウス通日を計算
                    dWkJd = dblGc2Jd(
                        stRes.iYear,
                        astHoliday[i].iMonth,
                        astHoliday[i].iDay,
                        0, 0, 0
                    );
                    // ユリウス通日から曜日を計算
                    iYobi = intCalcDayOfWeek(dWkJd);
                    // 値セット
                    astHoliday0[iSize0].iMonth = astHoliday[i].iMonth;
                    astHoliday0[iSize0].iDay   = astHoliday[i].iDay;
                    astHoliday0[iSize0].iKey   = astHoliday[i].iKey;
                    astHoliday0[iSize0].dJd    = dWkJd;
                    astHoliday0[iSize0].iYobi  = iYobi;
                    iSize0++;
                } else {                         // 月日が不定のもの
                    if (astHoliday[i].iKbn == 21) {  // 第2月曜日 ( 8 - 14 の月曜日)
                        for (int day = 8; day <= 14; day++) {
                            // グレゴリオ暦からユリウス通日を計算
                            dWkJd = dblGc2Jd(
                                stRes.iYear,
                                astHoliday[i].iMonth,
                                day,
                                0, 0, 0
                            );
                            // ユリウス通日から曜日を計算
                            iYobi = intCalcDayOfWeek(dWkJd);
                            if (iYobi == 1) {
                                // 値セット
                                astHoliday0[iSize0].iMonth = astHoliday[i].iMonth;
                                astHoliday0[iSize0].iDay   = day;
                                astHoliday0[iSize0].iKey   = astHoliday[i].iKey;
                                astHoliday0[iSize0].dJd    = dWkJd;
                                astHoliday0[iSize0].iYobi  = 1;
                                iSize0++;
                            }
                        }
                    } else if (astHoliday[i].iKbn == 31) {  // 第3月曜日 ( 15 - 21 の月曜日)
                        for (int day = 15; day <= 21; day++) {
                            // グレゴリオ暦からユリウス通日を計算
                            dWkJd = dblGc2Jd(
                                stRes.iYear,
                                astHoliday[i].iMonth,
                                day,
                                0, 0, 0
                            );
                            // ユリウス通日から曜日を計算
                            iYobi = intCalcDayOfWeek(dWkJd);
                            if (iYobi == 1) {
                                // 値セット
                                astHoliday0[iSize0].iMonth = astHoliday[i].iMonth;
                                astHoliday0[iSize0].iDay   = day;
                                astHoliday0[iSize0].iKey   = astHoliday[i].iKey;
                                astHoliday0[iSize0].dJd    = dWkJd;
                                astHoliday0[iSize0].iYobi  = 1;
                                iSize0++;
                            }
                        }
                    } else if (astHoliday[i].iKbn == 80) {  // 春分の日
                        // グレゴリオ暦からユリウス通日を計算
                        dWkJd = dblGc2Jd(
                            stRes.iYear,
                            astHoliday[i].iMonth,
                            31,
                            0, 0, 0
                        );
                        // 3月末直前の二分二至から春分の日を計算
                        if (!blnCalcLastNibunChu(dWkJd, 90)) {
                            pcErrMsg = pacMsg[8];
                        }
                        // 世界時計算
                        if (!blnCalcJd2Ymdt(dLastNibunChuTime)) {
                            pcErrMsg = pacMsg[7];
                        }
                        // ユリウス通日から曜日を計算
                        dWkJd = dblGc2Jd(
                            stRes.iYear,
                            astHoliday[i].iMonth,
                            aiYmdt[2],
                            0, 0, 0
                        );
                        iYobi = intCalcDayOfWeek(dWkJd);
                        // 値セット
                        astHoliday0[iSize0].iMonth = astHoliday[i].iMonth;
                        astHoliday0[iSize0].iDay   = aiYmdt[2];
                        astHoliday0[iSize0].iKey   = astHoliday[i].iKey;
                        astHoliday0[iSize0].dJd    = dWkJd;
                        astHoliday0[iSize0].iYobi  = iYobi;
                        iSize0++;
                    } else if (astHoliday[i].iKbn == 81) {  // 秋分の日
                        // グレゴリオ暦からユリウス通日を計算
                        dWkJd = dblGc2Jd(
                            stRes.iYear,
                            astHoliday[i].iMonth,
                            30,
                            0, 0, 0
                        );
                        // 9月末直前の二分二至から秋分の日を計算
                        if (!blnCalcLastNibunChu(dWkJd, 90)) {
                            pcErrMsg = pacMsg[8];
                        }
                        // 世界時計算
                        if (!blnCalcJd2Ymdt(dLastNibunChuTime)) {
                            pcErrMsg = pacMsg[7];
                        }
                        // ユリウス通日から曜日を計算
                        dWkJd = dblGc2Jd(
                            stRes.iYear,
                            astHoliday[i].iMonth,
                            aiYmdt[2],
                            0, 0, 0
                        );
                        iYobi = intCalcDayOfWeek(dWkJd);
                        // 値セット
                        astHoliday0[iSize0].iMonth = astHoliday[i].iMonth;
                        astHoliday0[iSize0].iDay   = aiYmdt[2];
                        astHoliday0[iSize0].iKey   = astHoliday[i].iKey;
                        astHoliday0[iSize0].dJd    = dWkJd;
                        astHoliday0[iSize0].iYobi  = iYobi;
                        iSize0++;
                    }
                }
            }
        }

        // 国民の休日計算
        // ( 「国民の祝日」で前後を挟まれた「国民の祝日」でない日 )
        // ( 年またぎは考慮していない(今のところ不要) )
        for (int i = 0; i <= iSize0 - 2; i++) {
            if ((int)(astHoliday0[i].dJd + 2) == (int)astHoliday0[i + 1].dJd) {
                dWkJd = astHoliday0[i].dJd + 1;
                if (astHoliday0[i].iYobi + 1 == 7) {
                    iYobi = 0;
                } else {
                    iYobi = astHoliday0[i].iYobi + 1;
                }
                // 世界時計算
                if (!blnCalcJd2Ymdt(dWkJd)) {
                    pcErrMsg = pacMsg[7];
                }
                // 値セット
                astHoliday1[iSize1].iMonth = aiYmdt[1];
                astHoliday1[iSize1].iDay   = aiYmdt[2];
                astHoliday1[iSize1].iKey   = 90;
                astHoliday1[iSize1].dJd    = dWkJd;
                astHoliday1[iSize1].iYobi  = iYobi;
                iSize1++;
            }
        }

        // 振替休日計算
        // ( 「国民の祝日」が日曜日に当たるときは、
        //   その日後においてその日に最も近い「国民の祝日」でない日 )
        for (int i = 0; i <= iSize0 - 1; i++) {
            if (astHoliday0[i].iYobi == 0) {
                dJdNext = astHoliday0[i].dJd + 1;
                if ((astHoliday0[i].iYobi + 1) == 7) {
                    iYobiNext = 0;
                } else {
                    iYobiNext = astHoliday0[i].iYobi + 1;
                }
                if (i == iSize0 - 1) {
                    // 世界時計算
                    if (!blnCalcJd2Ymdt(dJdNext)) {
                        pcErrMsg = pacMsg[7];
                    }
                    // 値セット
                    astHoliday2[iSize2].iMonth = aiYmdt[1];
                    astHoliday2[iSize2].iDay   = aiYmdt[2];
                    astHoliday2[iSize2].iKey   = 91;
                    astHoliday2[iSize2].dJd    = dJdNext;
                    astHoliday2[iSize2].iYobi  = iYobiNext;
                    iSize2++;
                } else {
                    int iFlgFurikae = 0;
                    int iDayPlus = 1;
                    while (iFlgFurikae == 0) {
                        if (i + iDayPlus < iSize0) {
                            if (dJdNext == astHoliday0[i + iDayPlus].dJd) {
                                dJdNext++;
                                if ((iYobiNext + 1) == 7) {
                                    iYobiNext = 0;
                                } else {
                                    iYobiNext = iYobiNext + 1;
                                }
                                iDayPlus++;
                            } else {
                                iFlgFurikae = 1;
                                // 世界時計算
                                if (!blnCalcJd2Ymdt(dJdNext)) {
                                    pcErrMsg = pacMsg[7];
                                }
                                // 値セット
                                astHoliday2[iSize2].iMonth = aiYmdt[1];
                                astHoliday2[iSize2].iDay   = aiYmdt[2];
                                astHoliday2[iSize2].iKey   = 91;
                                astHoliday2[iSize2].dJd    = dJdNext;
                                astHoliday2[iSize2].iYobi  = iYobiNext;
                                iSize2++;
                            }
                        }
                    }
                }
            }
        }

        // astHoliday0, astHoliday1, astHoliday2 を結合
        for (int i = 0; i < iSize0; i++) {
            astHolidayRes[iSize].iMonth = astHoliday0[i].iMonth;
            astHolidayRes[iSize].iDay   = astHoliday0[i].iDay;
            astHolidayRes[iSize].iKey   = astHoliday0[i].iKey;
            astHolidayRes[iSize].dJd    = astHoliday0[i].dJd;
            astHolidayRes[iSize].iYobi  = astHoliday0[i].iYobi;
            iSize++;
        }
        for (int i = 0; i < iSize1; i++) {
            astHolidayRes[iSize].iMonth = astHoliday1[i].iMonth;
            astHolidayRes[iSize].iDay   = astHoliday1[i].iDay;
            astHolidayRes[iSize].iKey   = astHoliday1[i].iKey;
            astHolidayRes[iSize].dJd    = astHoliday1[i].dJd;
            astHolidayRes[iSize].iYobi  = astHoliday1[i].iYobi;
            iSize++;
        }
        for (int i = 0; i < iSize2; i++) {
            astHolidayRes[iSize].iMonth = astHoliday2[i].iMonth;
            astHolidayRes[iSize].iDay   = astHoliday2[i].iDay;
            astHolidayRes[iSize].iKey   = astHoliday2[i].iKey;
            astHolidayRes[iSize].dJd    = astHoliday2[i].dJd;
            astHolidayRes[iSize].iYobi  = astHoliday2[i].iYobi;
            iSize++;
        }

        // 休日データ取得
        iHoliday = 99;
        for (int i = 0; i < iSize; i++) {
            if (astHolidayRes[i].iMonth == stRes.iMonth &&
                astHolidayRes[i].iDay   == stRes.iDay) {
                iHoliday = astHolidayRes[i].iKey;
                break;
            }
        }

        return true;
    } catch (...) {
        return false;
    }
}

/*
 * 旧暦取得処理
 */
bool Calendar::blnGetOldCalendar()
{
    try {
        // 旧暦計算
        if (!blnCalcOldCalendar(stRes.dJd)) {
            pcErrMsg = pacMsg[6];
            return false;
        }

        // 値セット
        stRes.dKokeiSun  = dKokeiSun;
        stRes.dKokeiMoon = dKokeiMoon;
        stRes.dMoonAge   = dMoonAge;
        stRes.iOcYear    = iOcYear;
        stRes.iOcLeap    = iOcLeap;
        stRes.iOcMonth   = iOcMonth;
        stRes.iOcDay     = iOcDay;
        stRes.iRokuyo    = iRokuyo;
        stRes.iKanshi    = iKanshi;
        stRes.iSekku     = iSekku;

        return true;
    } catch (...) {
        return false;
    }
}

/*
 * 旧暦計算
 */
bool Calendar::blnCalcOldCalendar(double dJd)
{
    double tm1, tm2, t;

    try {
        // 時刻引数を分解
        tm1 = floor(dJd);                  // 整数部分
        tm2 = dJd - tm1;                   // 小数部分
        tm2 -= 9.0/24.0;                   // JST ==> DT （補正時刻=0.0sec と仮定して計算）
        t  = (tm2 + 0.5) / 36525.0;        // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545.0) / 36525.0;  // 2451545は基点までのユリウス日
        //cout << "\tt          = " << t << endl;

        // ユリウス通日から黄経(太陽)を計算
        dKokeiSun = dblCalcLongitudeSun(t);

        // ユリウス通日から黄経(月)を計算
        dKokeiMoon = dblCalcLongitudeMoon(t);

        // ユリウス通日から月齢(正午)を計算
        dMoonAge = dblCalcMoonAgeNoon(dJd);

        // 旧暦計算(コア部分)
        if (!blnCalcOC(dJd)) {
            pcErrMsg = pacMsg[9];
            return false;
        }

        // 六曜計算
        if (!blnCalcRokuyo(iOcMonth, iOcDay)) {
            pcErrMsg = pacMsg[10];
            return false;
        }

        // 干支計算
        if (!blnCalcKanshi(dJd)) {
            pcErrMsg = pacMsg[11];
            return false;
        }

        // 節句計算
        if (!blnCalcSekku()) {
            pcErrMsg = pacMsg[12];
            return false;
        }

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  ユリウス日(JD)から旧暦を求める。
 *
 *    [ 引数 ]
 *      jd : ユリウス通日
 *
 *    [ 戻り値 ]
 *      真偽
 *
 *        aiOC[0] : 旧暦年
 *        aiOC[1] : 平月／閏月 flag .... 平月:0 閏月:1
 *        aiOC[2] : 旧暦月
 *        aiOC[3] : 旧暦日
 ******************************************************************************/
bool Calendar::blnCalcOC(double dJd)
{
    double adChuTime[4];   // 二分二至,中気の時刻
    int    aiChuKokei[4];  // 二分二至,中気の黄経
    double adSaku[5];      // 朔用配列宣言
    struct stM {           // 朔日行列構造体
        int    iMonth;     //   - 月名 (1:正月, 2:２月 ...)
        int    iLeap;      //   - 閏フラグ (0:平月, 1:閏月)
        double dJd;        //   - 朔日のjd
    };
    stM astM[5];           // 朔日行列
    double tm0, tm;        // 時刻計算用

    try {
        // 引数取得
        tm0 = dJd;

        // 計算対象の直前にあたる二分二至の時刻を計算
        //   adChuTime[0]  : 二分二至の時刻
        //   adChuKokei[0] : その時の太陽黄経
        if (!blnCalcLastNibunChu(tm0, 90)) {
            pcErrMsg = pacMsg[8];
        }
        adChuTime[0] = dLastNibunChuTime;
        aiChuKokei[0] = (int)dLastNibunChuKokei;

        // 中気の時刻を計算 ( 3回計算する )
        //   adChuTime[i][0]  : 中気の時刻
        //   adChuKokei[i][1] : その時の太陽黄経
        for (int i = 1; i <= 3; i++) {
            if (!blnCalcLastNibunChu(adChuTime[i - 1] + 32, 30)) {
                pcErrMsg = pacMsg[8];
            }
            adChuTime[i] = dLastNibunChuTime;
            aiChuKokei[i] = (int)dLastNibunChuKokei;
        }

        // 計算対象の直前にあたる二分二至の直前の朔の時刻を求める
        adSaku[0] = dblCalcLastSaku(adChuTime[0]);

        // 朔の時刻を求める
        for (int i = 1; i <= 4; i++) {
            tm  = adSaku[i-1];
            tm += 30;
            adSaku[i] = dblCalcLastSaku(tm);
            // 前と同じ時刻を計算した場合( 両者の差が26日以内 )には、初期値を
            // +33日にして再実行させる。
            if (fabs(floor(adSaku[i-1]) - floor(adSaku[i])) <= 26) {
                adSaku[i] = dblCalcLastSaku(adSaku[i-1] + 35);
            }
        }

        // saku[1]が二分二至の時刻以前になってしまった場合には、朔をさかのぼり過ぎ
        // たと考えて、朔の時刻を繰り下げて修正する。
        // その際、計算もれ（saku[4]）になっている部分を補うため、朔の時刻を計算
        // する。（近日点通過の近辺で朔があると起こる事があるようだ...？）
        if (floor(adSaku[1]) <= floor(adChuTime[0])) {
            for (int i = 0; i <= 3;i++) {
                adSaku[i] = adSaku[i + 1];
            }
            adSaku[4] = dblCalcLastSaku(adSaku[3] + 35);
        // saku[0]が二分二至の時刻以後になってしまった場合には、朔をさかのぼり足
        // りないと見て、朔の時刻を繰り上げて修正する。
        // その際、計算もれ（saku[0]）になっている部分を補うため、朔の時刻を計算
        // する。（春分点の近辺で朔があると起こる事があるようだ...？）
        } else if (floor(adSaku[0]) > floor(adChuTime[0])) {
            for (int i = 4; i <= 1; i++) {
                adSaku[i] = adSaku[i - 1];
            }
            adSaku[0] = dblCalcLastSaku(adSaku[0] - 27);
        }

        // 閏月検索Ｆｌａｇセット
        // （節月で４ヶ月の間に朔が５回あると、閏月がある可能性がある。）
        //   iLeap=0:平月  iLeap=1:閏月
        int iLeap;
        if (floor(adSaku[4]) <= floor(adChuTime[3])) {
            iLeap = 1;
        } else {
            iLeap = 0;
        }

        // 朔日行列の作成
        astM[0].iMonth = floor(aiChuKokei[0] / 30.0) + 2;
        if (astM[0].iMonth > 12) {
            astM[0].iMonth -= 12;
        }
        astM[0].iLeap = 0;
        astM[0].dJd = floor(adSaku[0]);
        for (int i = 1; i <= 4; i++) {
            if (iLeap == 1 && i != 1) {
                if (floor(adChuTime[i-1]) <= floor(adSaku[i-1]) ||
                    floor(adChuTime[i-1]) >= floor(adSaku[i])) {
                    astM[i-1].iMonth = astM[i-2].iMonth;
                    astM[i-1].iLeap  = 1;
                    astM[i-1].dJd    = floor(adSaku[i-1]);
                    iLeap = 0;
                }
            }
            astM[i].iMonth = astM[i-1].iMonth + 1;
            if (astM[i].iMonth > 12) {
                astM[i].iMonth -= 12;
            }
            astM[i].iLeap = 0;
            astM[i].dJd = floor(adSaku[i]);
        }

        // 朔日行列から旧暦を求める。
        int iState = 0;
        int iIndex = 0;
        for (int i = 0; i <= 4; i++) {
            iIndex = i;
            if (floor(tm0) < floor(astM[i].dJd)) {
                iState = 1;
                break;
            } else if (floor(tm0) == floor(astM[i].dJd)) {
                iState = 2;
                break;
            }
        }
        //if (iState == 0 || iState == 1) {
        if (iState == 1) {
            iIndex--;
        }
        iOcLeap  = astM[iIndex].iLeap;
        iOcMonth = astM[iIndex].iMonth;
        iOcDay   = floor(tm0) - floor(astM[iIndex].dJd) + 1;

        // 旧暦年の計算
        // （旧暦月が10以上でかつ新暦月より大きい場合には、
        //   まだ年を越していないはず...）
        if (!blnCalcJd2Ymdt(tm0)) {
            pcErrMsg = pacMsg[7];
        }
        iOcYear = aiYmdt[0];
        if (iOcMonth > 9 && iOcMonth > aiYmdt[1]) {
            iOcYear--;
        }

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  月日(旧暦)から六曜を計算する
 *
 *    旧暦一日の六曜
 *      １・７月   : 先勝
 *      ２・８月   : 友引
 *      ３・９月   : 先負
 *      ４・１０月 : 仏滅
 *      ５・１１月 : 大安
 *      ６・１２月 : 赤口
 *    と決まっていて、あとは月末まで順番通り。
 *    よって、月と日をたした数を６で割った余りによって六曜を決定することができます。
 *    ( 旧暦の月 ＋ 旧暦の日 ) ÷ 6 ＝ ？ … 余り
 *    余り 0 : 大安
 *         1 : 赤口
 *         2 : 先勝
 *         3 : 友引
 *         4 : 先負
 *         5 : 仏滅
 *
 *    [ 引数 ]
 *      iOcMonth : 旧暦の月
 *      iOcDay   : 旧暦の日
 *
 *    [ 戻り値 ]
 *      iRokuyo ( 六曜 ( 0 - 5 ) )
 ******************************************************************************/
bool Calendar::blnCalcRokuyo(int iOcMonth, int iOcDay)
{
    try {
        // 六曜計算
        // ( 六曜 = ( 旧暦の月 ＋ 旧暦の日 ) % 6 )
        iRokuyo = (iOcMonth + iOcDay) % 6;

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  ユリウス通日から干支(日)を計算する
 *   [ 引数 ]
 *     dJd : ユリウス通日
 *   [ 戻り値 ]
 *     iKanshi ( 干支 ( 0(甲子) - 59(癸亥) ) )
 *     ※[ ユリウス通日 - 10日 ] を60で割った剰余
 ******************************************************************************/
bool Calendar::blnCalcKanshi(double dJd)
{
    try {
        // 干支(日)計算
        iKanshi = ((int)dJd - 10) % 60;

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  引数で与えられた日付(グレゴリオ暦[月,日])の節句を求める
 *    [ 引数 ]
 *      なし
 *    [ 戻り値 ]
 *      iSekku : 節句のコード
 *    ※基点ユリウス日(2451545)：2000/1/2/0/0/0(年/月/日/時/分/秒…世界時)
 ******************************************************************************/
bool Calendar::blnCalcSekku()
{
    try {
        iSekku = 9;
        for (int i = 0; i < sizeof(astSekku) / sizeof(astSekku[0]); i++) {
            if (astSekku[i].iMonth == stRes.iMonth &&
                astSekku[i].iDay   == stRes.iDay) {
                iSekku = i;
                break;
            }
        }

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  年月日(グレゴリオ暦)からユリウス日(JD)を計算する
 *
 *    ﾌﾘｰｹﾞﾙの公式を使用する
 *    [ JD ] = int( 365.25 × year )
 *           + int( year / 400 )
 *           - int( year / 100 )
 *           + int( 30.59 ( month - 2 ) )
 *           + day
 *           + 1721088
 *    ※上記の int( x ) は厳密には、x を超えない最大の整数
 *      ( ちなみに、[ 準JD ]を求めるなら + 1721088 が - 678912 となる )
 *
 *    [ 戻り値 ]
 *      jd ( ユリウス日 )
 ******************************************************************************/
double Calendar::dblGc2Jd(int iY, int iM, int iD, int iHo, int iMi, int iSe)
{
    double dJd, dT;

    try {
        // 1月,2月は前年の13月,14月とする
        if (iM < 3) {
          iY -= 1;
          iM += 12;
        }

        // 日付(整数)部分計算
        dJd  = floor(365.25 * iY);
        dJd += floor(iY / 400.0);
        dJd -= floor(iY / 100.0);
        dJd += floor(30.59 * (iM - 2));
        dJd += iD;
        dJd += 1721088;

        // 時間(小数)部分計算
        dT  = iSe / 3600.0;
        dT += iMi / 60.0;
        dT += iHo;
        dT  = dT / 24.0;

        return (dJd + dT);
    } catch (...) {
        cout << pacMsg[13] << endl;
    }
}

/******************************************************************************
 *  ユリウス日(JD)から年月日、時分秒(世界時)を計算する
 *
 *    [ 引数 ]
 *      jd : ユリウス通日
 *
 *    [ 戻り値 ]
 *      真偽
 *
 *      iYmdt[0] ... 年
 *      iYmdt[1] ... 月
 *      iYmdt[2] ... 日
 *      iYmdt[3] ... 時
 *      iYmdt[4] ... 分
 *      iYmdt[5] ... 秒
 *
 *    ※ この関数で求めた年月日は、グレゴリオ暦法によって表されている。
 ******************************************************************************/
bool Calendar::blnCalcJd2Ymdt(double dJd)
{
    int x[7];
    int dTm;

    try {
        x[0] = floor(dJd + 68570);
        x[1] = floor(x[0] / 36524.25);
        x[2] = x[0] - floor(36524.25 * x[1] + 0.75);
        x[3] = floor((x[2] + 1) / 365.2425);
        x[4] = x[2] - floor(365.25 * x[3]) + 31;
        x[5] = floor(floor(x[4]) / 30.59);
        x[6] = floor(floor(x[5]) / 11.0);

        aiYmdt[2] = x[4] - floor(30.59 * x[5]);
        aiYmdt[1] = x[5] - 12 * x[6] + 2;
        aiYmdt[0] = 100 * (x[1] - 49) + x[3] + x[6];

        // 2月30日の補正
        if (aiYmdt[1] == 2 && aiYmdt[2] > 28) {
            if (aiYmdt[0] % 100 == 0 && aiYmdt[0] % 400 == 0) {
                aiYmdt[2] = 29;
            } else if (aiYmdt[0] % 4 == 0) {
                aiYmdt[2] = 29;
            } else {
                aiYmdt[2] = 28;
            }
        }

        dTm = 86400 * (dJd - floor(dJd));
        aiYmdt[3] = floor(dTm / 3600.0);
        aiYmdt[4] = floor((dTm - 3600 * aiYmdt[3]) / 60.0);
        aiYmdt[5] = floor(dTm - 3600 * aiYmdt[3] - 60 * aiYmdt[4]);

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  太陽の黄経 λsun を計算する
 ******************************************************************************/
double Calendar::dblCalcLongitudeSun(double dT)
{
    double dAng, dTh;

    try {
        // 摂動項の計算
        dAng = dblNormalizeAngle( 31557.0 * dT + 161.0);
        dTh  = 0.0004 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 29930.0 * dT +  48.0);
        dTh += 0.0004 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  2281.0 * dT + 221.0);
        dTh += 0.0005 * cos(dK * dAng);
        dAng = dblNormalizeAngle(   155.0 * dT + 118.0);
        dTh += 0.0005 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 33718.0 * dT + 316.0);
        dTh += 0.0006 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  9038.0 * dT +  64.0);
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  3035.0 * dT + 110.0);
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 65929.0 * dT +  45.0);
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 22519.0 * dT + 352.0);
        dTh += 0.0013 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 45038.0 * dT + 254.0);
        dTh += 0.0015 * cos(dK * dAng);
        dAng = dblNormalizeAngle(445267.0 * dT + 208.0);
        dTh += 0.0018 * cos(dK * dAng);
        dAng = dblNormalizeAngle(    19.0 * dT + 159.0);
        dTh += 0.0018 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 32964.0 * dT + 158.0);
        dTh += 0.0020 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 71998.1 * dT + 265.1);
        dTh += 0.0200 * cos(dK * dAng);
        dAng = dblNormalizeAngle(35999.05 * dT + 267.52);
        dTh -= 0.0048 * dT * cos(dK * dAng);
        dTh += 1.9147     * cos(dK * dAng);

        // 比例項の計算
        dAng = dblNormalizeAngle(36000.7695 * dT);
        dAng = dblNormalizeAngle(dAng + 280.4659);
        dTh  = dblNormalizeAngle(dTh + dAng);

        return dTh;
    } catch (...) {
        cout << pacMsg[14] << endl;
    }
}

/******************************************************************************
 *  月の黄経 λsun を計算する
 ******************************************************************************/
double Calendar::dblCalcLongitudeMoon(double dT)
{
    double dAng, dTh;

    try {
        // 摂動項の計算
        dAng = dblNormalizeAngle(2322131.0  * dT + 191.0 );
        dTh  = 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle(   4067.0  * dT +  70.0 );
        dTh += 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 549197.0  * dT + 220.0 );
        dTh += 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1808933.0  * dT +  58.0 );
        dTh += 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 349472.0  * dT + 337.0 );
        dTh += 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 381404.0  * dT + 354.0 );
        dTh += 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 958465.0  * dT + 340.0 );
        dTh += 0.0003 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  12006.0  * dT + 187.0 );
        dTh += 0.0004 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  39871.0  * dT + 223.0 );
        dTh += 0.0004 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 509131.0  * dT + 242.0 );
        dTh += 0.0005 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1745069.0  * dT +  24.0 );
        dTh += 0.0005 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1908795.0  * dT +  90.0 );
        dTh += 0.0005 * cos(dK * dAng);
        dAng = dblNormalizeAngle(2258267.0  * dT + 156.0 );
        dTh += 0.0006 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 111869.0  * dT +  38.0 );
        dTh += 0.0006 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  27864.0  * dT + 127.0 );
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 485333.0  * dT + 186.0 );
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 405201.0  * dT +  50.0 );
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 790672.0  * dT + 114.0 );
        dTh += 0.0007 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1403732.0  * dT +  98.0 );
        dTh += 0.0008 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 858602.0  * dT + 129.0 );
        dTh += 0.0009 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1920802.0  * dT + 186.0 );
        dTh += 0.0011 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1267871.0  * dT + 249.0 );
        dTh += 0.0012 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1856938.0  * dT + 152.0 );
        dTh += 0.0016 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 401329.0  * dT + 274.0 );
        dTh += 0.0018 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 341337.0  * dT +  16.0 );
        dTh += 0.0021 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  71998.0  * dT +  85.0 );
        dTh += 0.0021 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 990397.0  * dT + 357.0 );
        dTh += 0.0021 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 818536.0  * dT + 151.0 );
        dTh += 0.0022 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 922466.0  * dT + 163.0 );
        dTh += 0.0023 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  99863.0  * dT + 122.0 );
        dTh += 0.0024 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1379739.0  * dT +  17.0 );
        dTh += 0.0026 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 918399.0  * dT + 182.0 );
        dTh += 0.0027 * cos(dK * dAng);
        dAng = dblNormalizeAngle(   1934.0  * dT + 145.0 );
        dTh += 0.0028 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 541062.0  * dT + 259.0 );
        dTh += 0.0037 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1781068.0  * dT +  21.0 );
        dTh += 0.0038 * cos(dK * dAng);
        dAng = dblNormalizeAngle(    133.0  * dT +  29.0 );
        dTh += 0.0040 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1844932.0  * dT +  56.0 );
        dTh += 0.0040 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1331734.0  * dT + 283.0 );
        dTh += 0.0040 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 481266.0  * dT + 205.0 );
        dTh += 0.0050 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  31932.0  * dT + 107.0 );
        dTh += 0.0052 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 926533.0  * dT + 323.0 );
        dTh += 0.0068 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 449334.0  * dT + 188.0 );
        dTh += 0.0079 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 826671.0  * dT + 111.0 );
        dTh += 0.0085 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1431597.0  * dT + 315.0 );
        dTh += 0.0100 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1303870.0  * dT + 246.0 );
        dTh += 0.0107 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 489205.0  * dT + 142.0 );
        dTh += 0.0110 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1443603.0  * dT +  52.0 );
        dTh += 0.0125 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  75870.0  * dT +  41.0 );
        dTh += 0.0154 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 513197.9  * dT + 222.5 );
        dTh += 0.0304 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 445267.1  * dT +  27.9 );
        dTh += 0.0347 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 441199.8  * dT +  47.4 );
        dTh += 0.0409 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 854535.2  * dT + 148.2 );
        dTh += 0.0458 * cos(dK * dAng);
        dAng = dblNormalizeAngle(1367733.1  * dT + 280.7 );
        dTh += 0.0533 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 377336.3  * dT +  13.2 );
        dTh += 0.0571 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  63863.5  * dT + 124.2 );
        dTh += 0.0588 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 966404.0  * dT + 276.5 );
        dTh += 0.1144 * cos(dK * dAng);
        dAng = dblNormalizeAngle(  35999.05 * dT +  87.53);
        dTh += 0.1851 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 954397.74 * dT + 179.93);
        dTh += 0.2136 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 890534.22 * dT + 145.7 );
        dTh += 0.6583 * cos(dK * dAng);
        dAng = dblNormalizeAngle( 413335.35 * dT +  10.74);
        dTh += 1.2740 * cos(dK * dAng);
        dAng = dblNormalizeAngle(477198.868 * dT + 44.963);
        dTh += 6.2888 * cos(dK * dAng);

        // 比例項の計算
        dAng = dblNormalizeAngle(481267.8809 * dT);
        dAng = dblNormalizeAngle(dAng + 218.3162);
        dTh  = dblNormalizeAngle(dTh  +  dAng);

        return dTh;
    } catch (...) {
        cout << pacMsg[15] << endl;
    }
}

/******************************************************************************
 *  月齢(正午)を計算する
 *
 *    月齢は直前の朔の日時からの経過日数
 *
 *    [ 引数 ]
 *      jd : ユリウス通日
 *    [ 戻り値 ]
 *      moon_age ( 月齢 )
 ******************************************************************************/
double Calendar::dblCalcMoonAgeNoon(double dJd)
{
    double dLastSaku;
    double dMoonAgeNoon;

    try {
        // 直前の朔を計算
        dLastSaku = dblCalcLastSaku(dJd);

        // 月齢計算
        dMoonAgeNoon = dJd + 0.5 - dLastSaku;

        return dMoonAgeNoon;
    } catch (...) {
        cout << pacMsg[16] << endl;
    }
}

/******************************************************************************
 *  角度正規化
 *  ( すなわち引数の範囲を ０≦θ＜３６０ にする )
 ******************************************************************************/
double Calendar::dblNormalizeAngle(double dAngle)
{
    double dAngle1, dAngle2;

    try {
        if (dAngle < 0) {
            dAngle1  = dAngle * (-1);
            dAngle2  = floor(dAngle1 / 360.0);
            dAngle1 -= 360 * dAngle2;
            dAngle1  = 360 - dAngle1;
        } else {
            dAngle1  = floor(dAngle / 360.0);
            dAngle1  = dAngle - 360.0 * dAngle1;
        }

        return dAngle1;
    } catch (...) {
        cout << pacMsg[17] << endl;
    }
}

/******************************************************************************
 *  ユリウス通日(JD)から曜日(GC)を計算する
 *
 *   曜日 = ( ユリウス通日 + 2 ) % 7
 *      0 : 日曜
 *      1 : 月曜
 *      2 : 火曜
 *      3 : 水曜
 *      4 : 木曜
 *      5 : 金曜
 *      6 : 土曜
 *
 *    [ 引数 ]
 *      jd : ユリウス通日
 *
 *    [ 戻り値 ]
 *      yobi ( 曜日 ( 0 - 6 ) )
 ******************************************************************************/
int Calendar::intCalcDayOfWeek(double jd)
{
    int iYobi;

    try {
        // 曜日計算
        // ( 曜日 = ( ユリウス通日 + 2 ) % 7 )
        iYobi = ((int)jd + 2) % 7;

        return iYobi;
    } catch (...) {
        cout << pacMsg[18] << endl;
    }
}

/******************************************************************************
 *  ユリウス日(JD)から直前の二分二至、中気の時刻を求める
 *
 *    [ 引数 ]
 *      jd  : ユリウス日(JD)
 *      kbn : 90 ( 二分二至 ), 30 ( 中気 )
 *
 *    [ 戻り値 ]
 *      真偽
 *
 *      nibun_chu[0] : 二分二至、中気の時刻
 *      nibun_chu[1] : その時の黄経
 ******************************************************************************/
bool Calendar::blnCalcLastNibunChu(double dJd, int iKbn)
{
    double tm1, tm2, t;
    double rm_sun;
    int    rm_sun0;
    double delta_t1, delta_t2, delta_rm;

    try {
        // 時刻引数を分解
        tm1 = floor(dJd);  // 整数部分
        tm2 = dJd - tm1;   // 小数部分
        tm2 -= 9.0/24.0;   // JST ==> DT （補正時刻=0.0sec と仮定して計算）

        // 直前の二分二至の黄経 λsun0 を求める
        t  = (tm2 + 0.5) / 36525.0;        // 36525は1年365日と1/4を現す数値
        t += (tm1 - 2451545.0) / 36525.0;  // 2451545は基点までのユリウス日
        rm_sun = dblCalcLongitudeSun(t);
        rm_sun0 = iKbn * (double)(int)(rm_sun / iKbn);

        // 繰り返し計算によって直前の二分二至の時刻を計算する
        // （誤差が±1.0 sec以内になったら打ち切る。）
        delta_t1 = 0; delta_t2 = 1;
        while (fabs(delta_t1 + delta_t2) > (1.0 / 86400.0)) {
            // λsun を計算
            t  = (tm2 + 0.5) / 36525.0;      // 36525は1年365日と1/4を現す数値
            t += (tm1 - 2451545) / 36525.0;  // 2451545は基点までのユリウス日
            rm_sun = dblCalcLongitudeSun(t);

            // 黄経差 Δλ＝λsun －λsun0
            delta_rm = rm_sun - rm_sun0;

            // Δλの引き込み範囲（±180°）を逸脱した場合には、補正を行う
            if (delta_rm > 180.0) {
                delta_rm -= 360;
            } else if (delta_rm < -180.0) {
                delta_rm += 360;
            }

            // 時刻引数の補正値 Δt
            delta_t1 = (double)(int)(delta_rm * 365.2 / 360.0);
            delta_t2 = delta_rm * 365.2 / 360.0;
            delta_t2 -= delta_t1;

            // 時刻引数の補正
            tm1 = tm1 - delta_t1;
            tm2 = tm2 - delta_t2;
            if (tm2 < 0.0) {
                tm2 += 1;
                tm1 -= 1;
            }
        }

        // 戻り値の作成
        //   dNibunChuTime  : 時刻引数を合成するのと、DT ==> JST 変換を行い、戻り値とする
        //                  ( 補正時刻=0.0sec と仮定して計算 )
        //   dNibunChuKokei : 黄経
        dLastNibunChuTime  = tm2 + 9 / 24.0 + tm1;
        dLastNibunChuKokei = rm_sun0;

        return true;
    } catch (...) {
        return false;
    }
}

/******************************************************************************
 *  与えられた時刻の直近の朔の時刻（JST）を求める
 *
 *    [ 引数 ]
 *      jd  : 計算対象となる時刻 ( ユリウス日 )
 *
 *    [ 戻り値 ]
 *      saku : 朔の時刻
 *
 *   ※ 引数、戻り値ともユリウス日で表し、時分秒は日の小数で表す。
 ******************************************************************************/
double Calendar::dblCalcLastSaku(double dJd)
{
    int iLoopCount;
    double tm1, tm2, t;
    double delta_t1, delta_t2, delta_rm;
    double rm_sun, rm_moon;

    try {
        // LOOPカウンタのリセット
        iLoopCount = 1;

        // 時刻引数を分解する
        tm1 = floor(dJd);
        tm2 = dJd - tm1;
        tm2 -= 9 / 24.0;  // JST ==> DT ( 補正時刻=0.0sec と仮定して計算 )

        // 繰り返し計算によって朔の時刻を計算する
        // (誤差が±1.0 sec以内になったら打ち切る。)
        delta_t1 = 0; delta_t2 = 1;
        while (fabs(delta_t1 + delta_t2) > (1.0 / 86400.0)) {
            // 太陽の黄経λsun ,月の黄経λmoon を計算
            // t = (tm - 2451548.0 + 0.5)/36525.0
            t  = (tm2 + 0.5) / 36525.0;
            t += (tm1 - 2451545) / 36525.0;
            rm_sun  = dblCalcLongitudeSun(t);
            rm_moon = dblCalcLongitudeMoon(t);

            // 月と太陽の黄経差Δλ
            // Δλ＝λmoon－λsun
            delta_rm = rm_moon - rm_sun;

            // ﾙｰﾌﾟの1回目 ( lc = 1 ) で delta_rm < 0.0 の場合には引き込み範囲に
            // 入るように補正する
            if (iLoopCount == 1 && delta_rm < 0.0) {
                delta_rm = dblNormalizeAngle(delta_rm);

            //   春分の近くで朔がある場合 ( 0 ≦λsun≦ 20 ) で、月の黄経λmoon≧300 の
            //   場合には、Δλ＝ 360.0 － Δλ と計算して補正する
            } else if (rm_sun >= 0.0 && rm_sun <= 20.0 && rm_moon >= 300.0) {
                delta_rm = dblNormalizeAngle(delta_rm);
                delta_rm = 360 - delta_rm;

            // Δλの引き込み範囲 ( ±40° ) を逸脱した場合には、補正を行う
            } else if (abs(delta_rm) > 40.0) {
                delta_rm = dblNormalizeAngle(delta_rm);
            }

            // 時刻引数の補正値 Δt
            // delta_t = delta_rm * 29.530589 / 360.0;
            delta_t1  = (double)(int)(delta_rm * 29.530589 / 360.0);
            delta_t2  = delta_rm * 29.530589 / 360.0;
            delta_t2 -= delta_t1;

            // 時刻引数の補正
            // tm -= delta_t;
            tm1 -= delta_t1;
            tm2 -= delta_t2;
            if (tm2 < 0.0) {
                tm2 += 1;
                tm1 -= 1;
            }

            // ループ回数が15回になったら、初期値 tm を tm-26 とする。
            if (iLoopCount == 15 && abs(delta_t1 + delta_t2) > (1.0 / 86400.0)) {
                tm1 = floor(dJd - 26);
                tm2 = 0;

            // 初期値を補正したにも関わらず、振動を続ける場合には初期値を答えとして
            // 返して強制的にループを抜け出して異常終了させる。
            } else if (iLoopCount > 30 && abs(delta_t1 + delta_t2) > (1.0 / 86400.0)) {
                tm1 = dJd;
                tm2 = 0;
                break;
            }

            // LOOPカウンタインクリメント
            iLoopCount++;
        }

        // 時刻引数を合成するのと、DT ==> JST 変換を行い、戻り値とする
        // （補正時刻=0.0sec と仮定して計算）
        return (tm2 + tm1 + 9 / 24.0);
    } catch (...) {
        cout << pacMsg[19] << endl;
    }
}

/*
 * 結果出力処理
 */
bool Calendar::blnDisplay()
{
    string sOut;

    try {
        // 設定年月日
        printf("[%04d/%02d/%02d]", stRes.iYear, stRes.iMonth, stRes.iDay);

        // 各オプション計算値
        int i = 0;
        while (*pcOption) {
            switch (*pcOption) {
                case 'a':  // 曜日
                    printf(" %s曜日", pacWeek[stRes.iWeekDay]);
                    break;
                case 'b':  // 休日
                    if (stRes.iHoliday != 99) {
                        for (int j = 0; j < sizeof(astHoliday) / sizeof(astHoliday[0]); j++)
                            if (astHoliday[j].iKey == astHolidayRes[stRes.iHoliday].iKey)
                                printf(" %s", astHoliday[j].cName);
                    }
                    break;
                case 'c':  // ユリウス通日
                    printf(" %.1f", stRes.dJd + 0.5);
                    break;
                case 'd':  // 干支
                    printf(" %s%s", pacKan[stRes.iKanshi % 10], pacShi[stRes.iKanshi % 12]);
                    break;
                case 'e':  // 旧暦
                    printf(" %04d/%02d/%02d", stRes.iOcYear, stRes.iOcMonth, stRes.iOcDay);
                    break;
                case 'f':  // 六曜
                    printf(" %s", pacRokuyo[stRes.iRokuyo]);
                    break;
                case 'g':  // 二十四節気
                    if (stRes.iSekki24 != 999)
                        printf(" %s", pacSekki24[stRes.iSekki24 / 15]);
                    break;
                case 'h':  // 雑節
                    if (stRes.iZassetsu1 != 99) {
                        printf(" %s", pacZassetsu[stRes.iZassetsu1]);
                        if (stRes.iZassetsu2 != 99)
                            printf("・%s", pacZassetsu[stRes.iZassetsu2]);
                    }
                    break;
                case 'i':  // 節句
                    if (stRes.iSekku != 9) {
                        for (int j = 0; j < sizeof(astSekku) / sizeof(astSekku[0]); j++)
                            if (stRes.iSekku == j)
                                printf(" %s", astSekku[j].cName);
                    }
                    break;
                case 'j':  // 黄経(太陽)
                    printf(" %.4f", stRes.dKokeiSun);
                    break;
                case 'k':  // 黄経(月)
                    printf(" %.4f", stRes.dKokeiMoon);
                    break;
                case 'l':  // 月齢
                    printf(" %.4f", stRes.dMoonAge);
                    break;
            }
            i++;
            *pcOption++;
        }
        printf("\n");

        return true;
    } catch (...) {
        return false;
    }
}

/*
 * メイン処理
 */
bool Calendar::execProc()
{
    // グレゴリオ暦からユリウス通日を計算
    if (!blnGetJd()) {
        pcErrMsg = pacMsg[25];
        return false;
    }

    // 二十四節気取得
    if (!blnGetSekki24()) {
        pcErrMsg = pacMsg[20];
        return false;
    }

    // 雑節の取得
    if (!blnGetZassetsu()) {
        pcErrMsg = pacMsg[21];
        return false;
    }

    // 休日の取得
    if (!blnGetHoliday()) {
        pcErrMsg = pacMsg[22];
        return false;
    }

    // 旧暦の取得
    if (!blnGetOldCalendar()) {
        pcErrMsg = pacMsg[23];
        return false;
    }

    // 結果出力
    if (!blnDisplay()) {
        pcErrMsg = pacMsg[24];
        return false;
    }

    return true;
}

