/*
 * Calendar クラス
 */
class Calendar
{
    // ====< 定数定義 >====
    static const double PI = 3.141592653589793238462; // 円周率の定義

    // ====< 各種宣言 >====
    struct stRES {                                    // 結果格納用構造体
        int    iYear;                                 //   - 年
        int    iMonth;                                //   - 月
        int    iDay;                                  //   - 日
        double dJd;                                   //   - ユリウス通日
        int    iWeekDay;                              //   - 曜日
        int    iHoliday;                              //   - 休日
        int    iKanshi;                               //   - 干支
        int    iOcYear;                               //   - 旧暦・年
        int    iOcLeap;                               //   - 旧暦・閏月フラグ
        int    iOcMonth;                              //   - 旧暦・月
        int    iOcDay;                                //   - 旧暦・日
        int    iRokuyo;                               //   - 六曜
        int    iSekki24;                              //   - 二十四節気
        int    iZassetsu1;                            //   - 雑節１
        int    iZassetsu2;                            //   - 雑節２
        int    iSekku;                                //   - 節句
        double dKokeiSun;                             //   - 黄経(太陽)
        double dKokeiMoon;                            //   - 黄経(月)
        double dMoonAge;                              //   - 月齢
    };
    stRES stRes;                                      // 結果格納用

    double dK;                                        // 定義・度からラジアンに変換する係数

    const char *pcErrMsg;                             // エラーメッセージ
    const char *pcOption;                             // オプション

    const char* pacMsg[26];                           // メッセージマスタ配列ポインタ
    const char* pacWeek[7];                           // 曜日マスタ配列ポインタ
    const char* pacRokuyo[6];                         // 六曜マスタ配列ポインタ
    const char* pacKan[10];                           // 干マスタ配列ポインタ
    const char* pacShi[12];                           // 支マスタ配列ポインタ
    const char* pacSekki24[24];                       // 二十四節気マスタ配列ポインタ
    struct stSEKKU {
        int iMonth;
        int iDay;
        const char* cName;
    };                                                // 節句マスタ構造体
    stSEKKU astSekku[5];                              // 節句マスタ配列
    const char* pacZassetsu[18];                      // 雑節マスタ配列ポインタ
    struct stHOLIDAY {
        int iKey;
        int iMonth;
        int iDay;
        int iKbn;
        const char* cName;
    };                                                // 休日マスタ構造体
    stHOLIDAY astHoliday[17];                         // 休日マスタ配列
    struct stHOLIDAY_RES {
        int    iMonth;
        int    iDay;
        int    iKey;
        double dJd;
        int    iYobi;
    };                                                // 休日データ構造体
    stHOLIDAY_RES astHolidayRes[20];                  // 休日データ配列

    int    iYear, iMonth, iDay, iWeekDay;             // ワーク用・年, 月, 日, 曜日
    double dJd;                                       // ワーク用・ユリウス通日
    int    iSekki24;                                  // ワーク用・二十四節気
    int    iZassetsu1, iZassetsu2;                    // ワーク用・雑節
    int    iHoliday;                                  // ワーク用・休日
    int    iOcYear, iOcLeap, iOcMonth, iOcDay;        // ワーク用・旧暦(年, 閏月フラグ, 月, 日)
    int    iRokuyo;                                   // ワーク用・六曜
    int    iKanshi;                                   // ワーク用・干支
    int    iSekku;                                    // ワーク用・節句
    int    aiYmdt[6];                                 // ワーク用・世界時(年月日時分秒)
    double dLastNibunChuTime;                         // ワーク用・直近二分二至時刻
    double dLastNibunChuKokei;                        // ワーク用・直近二分二至黄経
    double dNibunChuTime;                             // ワーク用・二分二至時刻
    double dNibunChuKokei;                            // ワーク用・二分二至黄経
    double dKokeiSun, dKokeiMoon, dMoonAge;           // ワーク用・黄経(太陽, 月), 月齢

    void defMsg();                                    // 定義・メッセージ
    void defWeek();                                   // 定義・曜日
    void defRokuyo();                                 // 定義・六曜
    void defKanshi();                                 // 定義・干支
    void defSekki24();                                // 定義・二十四節気
    void defSekku();                                  // 定義・節句
    void defZassetsu();                               // 定義・雑節
    void defHoliday();                                // 定義・休日

    bool   blnGetJd();                                // 取得処理・ユリウス通日
    bool   blnGetSekki24();                           // 取得処理・二十四節気
    bool   blnGetZassetsu();                          // 取得処理・雑節
    bool   blnGetHoliday();                           // 取得処理・休日
    bool   blnGetOldCalendar();                       // 取得処理・旧暦
    bool   blnDisplay();                              // 結果出力処理

    bool   blnCalcSekki24(double dJd);                // 計算・二十四節気
    bool   blnCalcZassetsu(double dJd);               // 計算・雑節
    bool   blnCalcHoliday(double dJd);                // 計算・休日
    bool   blnCalcOldCalendar(double dJd);            // 計算・旧暦
    bool   blnCalcOC(double dJd);                     // 計算・旧暦(本質部分)
    bool   blnCalcRokuyo(int iOcMonth, int iOcDay);   // 計算・六曜
    bool   blnCalcKanshi(double dJd);                 // 計算・干支
    bool   blnCalcSekku();                            // 計算・節句
    double dblGc2Jd(int iY, int iM, int iD, int iHo, int iMi, int iSe);
                                                      // 計算・ユリウス日(GC->JD)
    bool   blnCalcJd2Ymdt(double dJd);                // 計算・世界時
    int    intCalcDayOfWeek(double dJd);              // 計算・曜日
    bool   blnCalcLastNibunChu(double dJd, int iKbn); // 計算・直近二分二至
    double dblCalcLastSaku(double dJd);               // 計算・直近朔
    double dblCalcLongitudeSun(double t);             // 計算・黄経(太陽)
    double dblCalcLongitudeMoon(double t);            // 計算・黄経(月)
    double dblCalcMoonAgeNoon(double dJd);            // 計算・月齢(正午)
    double dblNormalizeAngle(double dAngle);          // 計算・角度正規化

    //int    intGetSekki24();                           // 二十四節気取得
    //int    intGetZassetsu(int iKbn);                  // 雑節取得
    //int    intGetHoliday();                           // 休日取得
    //int    intGetYmdt(int iKbn);                      // 世界時取得

    bool isNumeric(char *num);                        // 数値チェック
    bool isChar(char *opt);                           // 半角英字チェック
    bool checkDate(int iYear, int iMonth, int iDay);  // 日付妥当性チェック
    bool isLeapYear(int iYear);                       // 閏年チェック
    int  daysInMonth(int iYear, int iMonth);          // 月内日数

    public:
        Calendar();                                   // コンストラクタ
        const char* getErrMsg();                      // エラーメッセージ取得
        bool checkArg(int argc, char *argv[]);        // 引数チェック
        bool execProc();                              // メイン処理
};

