// $ g++ main.cpp common.cpp calendar.cpp -o calendar
/*******************************************************************************
  カレンダー情報
  ：カレンダー情報を取得する

  date          name            version
  2012.11.15    mk-mode         1.00 新規作成

  Copyright(C) 2011-2012 mk-mode.com All Rights Reserved.
--------------------------------------------------------------------------------
  原典 : 「旧暦計算サンプルプログラム」
         Copyright (C) 1993,1994 by H.Takano
         http://www.vector.co.jp/soft/dos/personal/se016093.html

  ・このRubyスクリプトの計算結果は無保証です。
  ・このRubyスクリプトはフリーソフトであり、自由に再利用・改良を行ってかまいませ
    んが、旧暦計算アルゴリズム部分についての著作権は原典の jgAWK版を開発された高
    野英明氏に帰属しています。上記のリンクより高野氏の「QRSAMP」を取得し、そのド
    キュメント内に書かれている再配布規定に従ってください。
  ・旧暦計算アルゴリズム部分以外(休日・曜日・干支・二十四節気・雑節・節句・月齢)
    についての著作権は mk-mode.comに帰属します。個人的に再利用・改良を行ってもか
    まいませんし、再配布について一方的に拒否することはありませんが、念のためご一
    報だけください。( MAIL : postmaster@mk-mode.com )
--------------------------------------------------------------------------------
  引数 :  [ オプション ] a : 曜日
                         b : 休日
                         c : ユリウス通日
                         d : 干支
                         e : 旧暦
                         f : 六曜
                         g : 二十四節気
                         h : 雑節
                         i : 節句
                         j : 黄経(太陽)
                         k : 黄経(月)
                         l : 月齢
          上記の半角小文字アルファベットが指定可能です。
          無指定なら全指定と判断します。

          [ 計算日付 ]   新暦 ( グレゴリオ暦 ) [ 半角８桁数字 ]
              新暦(グレゴリオ暦)を半角８桁数字で指定します。
              無指定なら当日(システム日付)と判断します。

  引数パターン :  +------------+------------+
                  |  第１引数  |  第２引数  |
                  +------------+------------+
                  |     無     |     無     |
                  +------------+------------+
                  |  ８桁数字  |     無     |
                  +------------+------------+
                  |  半角英字  |     無     |
                  +------------+------------+
                  |  半角英字  |  ８桁数字  |
                  +------------+------------+
 ******************************************************************************/
#include <iostream>
using namespace std;

#include "calendar.h"
#include "common.h"

/*
 * メイン処理
 */
int main(int argc, char *argv[])
{
    try
    {
        // Common クラス
        Common objCmn;

        // Calendar クラス
        Calendar objCal;

        // 引数チェック
        if (!objCal.checkArg(argc, argv)) {
            objCmn.dispMessage(9, objCal.getErrMsg());
            return 0;
        }

        // メイン処理
        if (!objCal.execProc()) {
            objCmn.dispMessage(9, objCal.getErrMsg());
            return 0;
        }
    }
    catch (...) {
        cout << "例外発生！" << endl;
        return -1;
    }

    // 正常終了
    return 0;
}

